@@ -22,4 +22,7 @@ wave_grade = [0,0.01,0.1,0.5,1.3,2.5,4.0,6.0,9.0,14.0,100]
 cmadaas_set = None
 cimiss_set = None
 plot_color_dict = None
-ws_jar_path = None
\ No newline at end of file
+ws_jar_path = None
+height_oy = None
+z500_cli =  pkg_resources.resource_filename('meteva', "resources/z500_cli.nc")
+language = "ch"
\ No newline at end of file
