@@ -706,4 +706,8 @@ class shpfile:
     china = "NationalBorder"
     province = "Province"
     county = "BOUNT_poly"
-    river = "hyd1_4p"
\ No newline at end of file
+    river = "hyd1_4p"
+
+class dtime_units:
+    hour = "hour"
+    minute = "minute"
\ No newline at end of file
