@@ -160,8 +160,8 @@ def xarray_to_griddata(xr0,
         ds0 = xr.Dataset({'data0': xr0})
     else:
         if value_name is not None:
-            da = xr0[value_name]
-            ds0 = xr.Dataset({'data0': da})
+            da_ = xr0[value_name]
+            ds0 = xr.Dataset({'data0': da_})
             name = value_name
         else:
             ds0 = xr0
@@ -169,10 +169,14 @@ def xarray_to_griddata(xr0,
 
     if dtime_dim == "time":
         ds0 = ds0.rename_dims({"time": "dtime"})
+        if 'time' in ds0.coords:
+            ds0 = ds0.rename({'time': 'dtime'})
         dtime_dim = "dtime"
 
     if time_dim == "dtime":
         ds0 = ds0.rename_dims({"dtime": "time"})
+        if 'dtime' in ds0.coords:
+            ds0 = ds0.rename({'dtime': 'time'})
         time_dim = "time"
 
     drop_list = []
@@ -326,7 +330,6 @@ def xarray_to_griddata(xr0,
         ds.coords["lon"] = ("lon", [0])
 
 
-
     if da is None:
         name_list = list((ds0))
         for name in name_list:
@@ -372,28 +375,32 @@ def xarray_to_griddata(xr0,
 
     if "member" not in dim_order.keys():
         #print(da)
-
         dim_order["member"] = "member"
         da = da.expand_dims("member")
+        da = da.copy()
     if "time" not in dim_order.keys():
         dim_order["time"] = "time"
         da = da.expand_dims("time")
+        da = da.copy()
     if "level" not in dim_order.keys():
         dim_order["level"] = "level"
         da = da.expand_dims("level")
+        da = da.copy()
     if "dtime" not in dim_order.keys():
         dim_order["dtime"] = "dtime"
         da = da.expand_dims("dtime")
+        da = da.copy()
     if "lat" not in dim_order.keys():
         dim_order["lat"] = "lat"
         da = da.expand_dims("lat")
+        da = da.copy()
     if "lon" not in dim_order.keys():
         dim_order["lon"] = "lon"
         da = da.expand_dims("lon")
-
+        da = da.copy()
     da = da.transpose(dim_order["member"], dim_order["level"], dim_order["time"],
                       dim_order["dtime"], dim_order["lat"], dim_order["lon"])
-    #print(da)
+
     ds[name] = (("member", "level", "time", "dtime", "lat", "lon"), da.values)
     attrs_name = list(da.attrs)
     for key in attrs_name:
@@ -446,17 +453,22 @@ def xarray_to_griddata(xr0,
 
     meteva.base.reset(da1)
     lats = da1.lat.values
-    dlats = lats[1:] - lats[:-1]
-    maxdlats = np.max(dlats)
-    mindlats = np.min(dlats)
-    if (maxdlats - mindlats)/maxdlats > 0.001:
-        print("***")
-        nlat = int((lats[-1] - lats[0])/mindlats) + 2
-        dlat = (lats[-1] - lats[0])/(nlat-1)
-        lons = da1.lon.values
-        dlon = (lons[-1] - lons[0])/(len(lons) - 1)
-        grid = meteva.base.grid([lons[0],lons[-1],dlon],[lats[0],lats[-1],dlat])
-        da1 = meteva.base.interp_xg_linear(da1,grid)
+    lons = da1.lon.values
+    if len(lats)>1 and len(lons)>1:
+        dlats = lats[1:] - lats[:-1]
+        maxdlats = np.max(dlats)
+        mindlats = np.min(dlats)
+        if (maxdlats - mindlats)/maxdlats > 0.001:
+            print("***")
+            nlat = int((lats[-1] - lats[0])/mindlats) + 2
+            dlat = (lats[-1] - lats[0])/(nlat-1)
+            dlon = (lons[-1] - lons[0])/(len(lons) - 1)
+            grid = meteva.base.grid([lons[0],lons[-1],dlon],[lats[0],lats[-1],dlat])
+            try:
+                da1 = meteva.base.interp_xg_linear(da1,grid)
+            except:
+                #如果不能重置网格就还是返回原来的网格数据
+                pass
 
     return da1
 
@@ -877,11 +889,34 @@ def DataArray_to_grd(dataArray,member = None,level = None,time = None,dtime = No
 
 def reset(grd):
     lats = grd["lat"].values
+    if len(lats)>1:
+        if lats[0]>lats[1]:
+            lats = grd["lat"].values[::-1]
+            grd['lat'] = lats
+            dat = grd.values[:, :, :, :, ::-1, :]
+            grd.values = dat
 
-    if lats[0]>lats[1]:
-        lats = grd["lat"].values[::-1]
-        grd['lat'] = lats
-        dat = grd.values[:, :, :, :, ::-1, :]
-        grd.values = dat
+    lons = grd["lon"].values
+    if len(lons)>1:
+        if lons[0]>lons[1]:
+            lons = grd["lon"].values[::-1]
+            grd['lon'] = lons
+            dat = grd.values[:, :, :, :, :, ::-1]
+            grd.values = dat
 
-    return
\ No newline at end of file
+    return
+
+
+def set_griddata_attrs(grd, dtime_units = None,data_source = None,level_type =None,
+             var_name = None,var_cn_name = None,
+             var_units = None,valid_time = None,data_start_columns = None):
+
+    if grd.attrs is None: grd.attrs = {}
+    if dtime_units is not None:grd.attrs["dtime_units"] = dtime_units
+    if data_source is not None:grd.attrs["data_source"] = data_source
+    if level_type is not None: grd.attrs["data_type"] = level_type
+    if var_name is not None: grd.attrs["var_name"] = var_name
+    if var_cn_name is not None:grd.attrs["var_cn_name"] = var_cn_name
+    if var_units is not None:grd.attrs["var_units"] = var_units
+    if valid_time is not None:grd.attrs["valid_time"] = valid_time
+    if data_start_columns is not None:grd.attrs["data_start_columns"] = data_start_columns
