@@ -4,6 +4,8 @@ from scipy.spatial import cKDTree
 from scipy.interpolate import LinearNDInterpolator
 import copy
 import pandas as pd
+import math
+
 
 def reset_global_griddata(grd):
     '''
@@ -100,6 +102,7 @@ def interp_gs_nearest(grd,sta,used_coords = "xy"):
         #sta_all = meteva.base.combine_join(sta_all,sta)
         sta_all = pd.concat(sta_list,axis = 0)
         sta_all = sta_all.reindex(columns=column_list2)
+        sta_all.attrs = copy.deepcopy(grd.attrs)
         return sta_all
 
 
@@ -147,6 +150,7 @@ def interp_gs_linear(grd,sta,used_coords = "xy"):
         #sta_all = meteva.base.combine_join(sta_all,sta)
         sta_all = pd.concat(sta_list, axis=0)
         sta_all = sta_all.reindex(columns=column_list2)
+        sta_all.attrs = copy.deepcopy(grd.attrs)
         return sta_all
 
     elif used_coords == "xyz":
@@ -180,6 +184,7 @@ def interp_gs_linear(grd,sta,used_coords = "xy"):
                     sta.loc[:,members[m]] = dat_sta
                 sta_all = meteva.base.combine_join(sta_all,sta)
         sta_all = sta_all.reindex(columns=column_list2)
+        sta_all.attrs = copy.deepcopy(grd.attrs)
         return sta_all
 
     elif used_coords =="xydt":
@@ -218,6 +223,7 @@ def interp_gs_linear(grd,sta,used_coords = "xy"):
                     sta.loc[:, members[m]] = dat_s
                 sta_all = meteva.base.combine_join(sta_all,sta)
         sta_all = sta_all.reindex(columns=column_list2)
+        sta_all.attrs = copy.deepcopy(grd.attrs)
         return sta_all
 
 
@@ -260,6 +266,7 @@ def interp_gs_cubic(grd,sta,used_coords = "xy"):
                         sta.loc[:, members[m]] =sum
                     sta_all = meteva.base.combine_join(sta_all,sta)
         sta_all = sta_all.reindex(columns=column_list2)
+        sta_all.attrs = copy.deepcopy(grd.attrs)
         return sta_all
 
 
@@ -284,6 +291,7 @@ def interp_gs_cubic(grd,sta,used_coords = "xy"):
     sta1['dtime'] = grid.dtimes[0]
     sta1['level'] = grid.levels[0]
     meteva.base.basicdata.set_stadata_names(sta1,grid.members)
+    sta_all.attrs = copy.deepcopy(grd.attrs)
     return sta1
 
 
@@ -338,6 +346,7 @@ def interp_sg_idw(sta0, grid, background=None, effectR=1000, nearNum=8,decrease
         grd_list.append(grd)
 
     grd_all = meteva.base.concat(grd_list)
+    grd_all.attrs = copy.deepcopy(sta0.attrs)
     return grd_all
 
 
@@ -380,6 +389,7 @@ def interp_sg_idw_delta(sta0, grid,  halfR=1000, nearNum=8,decrease = 2):
         grd.name = data_name[0]
         grd_list.append(grd)
     grd_all = meteva.base.concat(grd_list)
+    grd_all.attrs = copy.deepcopy(sta0.attrs)
     return grd_all
 
 
@@ -430,6 +440,7 @@ def interp_sg_cressman(sta0, grid, r_list,background=None , nearNum=100):
         grd.name = data_name[0]
         grd_list.append(grd)
     grd_all = meteva.base.concat(grd_list)
+    grd_all.attrs = copy.deepcopy(sta0.attrs)
     return grd_all
 
 
@@ -459,9 +470,63 @@ def interp_ss_idw(sta0, station, effectR=1000, nearNum=8,decrease = 2,defalut_va
         dat = input_dat[inds]
         dat[:] = np.where(d[:] > effectR,defalut_value,dat[:])
         sta1.iloc[:,-1] = dat[:]
+    sta1.attrs = copy.deepcopy(sta0.attrs)
     return sta1
 
 
+def interp_gg_nearest(grd, grid,used_coords = "xy",outer_value = None):
+    '''
+    格点到格点插值
+    :param grd:左边的网格数据信息
+    :param grid :右边的网格数据信息
+    :other_info:网格数据除了xy方向的数值之外，还有time,dtime，leve member 等维度的值，如果other_info= 'left’则返回结果中这些维度的值就采用grd里的值，
+    否则采用grid里的值，默认为：left
+    :return:双线性插值之后的结果
+    '''
+    if (grd is None):
+        return None
+    levels = grd["level"].values
+    times = grd["time"].values
+    dtimes = grd["dtime"].values
+    members = grd["member"].values
+    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
+    iscycle = (grid0.dlon * grid0.nlon >= 360)
+    if used_coords == "xy":
+        is_out = False
+        if not iscycle:
+            if (grid.elon > grid0.elon or grid.slon < grid0.slon or grid.elat > grid0.elat or grid.slat < grid0.slat):
+                if outer_value is None:
+                    print("当目标网格超出数据网格时，outer_value参数必须赋值")
+                    return None
+                is_out = True
+
+        if is_out:
+            grid_new0 = meteva.base.get_inner_grid(grid,grid0)
+            grid_new = meteva.base.grid(grid_new0.glon, grid_new0.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
+        else:
+            grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
+        grd_new = meteva.base.grid_data(grid_new)
+        for i in range(len(levels)):
+            for j in range(len(times)):
+                for k in range(len(dtimes)):
+                    for m in range(len(members)):
+                        # 六维转换为二维的值
+                        dat = grd.values[m,i,j,k,:,:]
+                        #插值处理
+                        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
+                        ig = np.around(x[:]).astype(dtype='int16')
+                        y = (np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat - grid0.slat) / grid0.dlat
+                        jg = np.around(y[:]).astype(dtype='int16')
+                        ii, jj = np.meshgrid(ig, jg)
+                        dat2 = dat[jj, ii]
+                        grd_new.values[m,i,j,k,:,:] = dat2
+        if is_out:
+            grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
+            grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
+    grd_new.attrs = copy.deepcopy(grd.attrs)
+    return grd_new
+
+
 def interp_gg_linear(grd, grid,used_coords = "xy",outer_value = None):
     '''
     格点到格点插值
@@ -526,10 +591,10 @@ def interp_gg_linear(grd, grid,used_coords = "xy",outer_value = None):
         if is_out:
             grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
             grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
+    grd_new.attrs = copy.deepcopy(grd.attrs)
     return grd_new
 
 
-
 def interp_gg_cubic(grd, grid,used_coords = "xy",outer_value = None):
     '''
     格点到格点插值
@@ -601,6 +666,7 @@ def interp_gg_cubic(grd, grid,used_coords = "xy",outer_value = None):
         if is_out:
             grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
             grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
+    grd_new.attrs = copy.deepcopy(grd.attrs)
     return grd_new
 
 
@@ -668,3 +734,4 @@ def cubic_f(n, dx):
         return -(dx + 1) * dx * (dx - 2) / 2
     else:
         return (dx + 1) * dx * (dx - 1) / 6
+
