@@ -1,8 +1,6 @@
 import meteva
 import numpy as np
 import matplotlib.pyplot as plt
-plt.rcParams['font.sans-serif']=['SimHei'] #用来正常显示中文标签
-plt.rcParams['axes.unicode_minus']=False #用来正常显示负号
 import math
 from matplotlib.colors import BoundaryNorm
 
@@ -42,7 +40,8 @@ def add_map(ax,add_county_line = False,add_worldmap = True,title = None,sup_font
 
 def creat_axs(nplot,map_extend,ncol = None,height  = None,width = None,dpi = 300,sup_title = None,sup_fontsize = 12,
               add_county_line = False,add_worldmap = True,add_minmap = None,title_list = None,add_index = None,wspace = None,grid = True,
-              xticks_inter = None,yticks_inter = None,linewidth = [0.3,0.3,0.2],color = ["k","k","k"]):
+              xticks_inter = None,yticks_inter = None,linewidth = [0.3,0.3,0.2],color = ["k","k","k"],keep_ticks = 1,
+              width_colorbar = 0.5):
 
 
     ax_index = []
@@ -77,17 +76,23 @@ def creat_axs(nplot,map_extend,ncol = None,height  = None,width = None,dpi = 300
         sup_height_title = sup_fontsize * 0.12
 
     height_title = sup_fontsize * 0.1
-    height_bottem_xticsk = sup_fontsize * 0.05
-    height_hspace = sup_fontsize * 0.025
-
+    if keep_ticks==1:
+        height_hspace = sup_fontsize * 0.025
+    else:
+        height_hspace = sup_fontsize * 0.005
 
     if wspace is None:
         width_wspace = height_hspace*2
     else:
         width_wspace = wspace
 
-    width_colorbar = 0.5
-    width_left_yticks = sup_fontsize * 0.2
+
+    if keep_ticks==1:
+        width_left_yticks = sup_fontsize * 0.2
+        height_bottem_xticsk = sup_fontsize * 0.05
+    else:
+        width_left_yticks = 0
+        height_bottem_xticsk =0
 
     if ncol is None:
         match_list = []
@@ -224,12 +229,24 @@ def creat_axs(nplot,map_extend,ncol = None,height  = None,width = None,dpi = 300
         ax.set_xlim((slon, elon))
         ax.set_ylim((slat, elat))
 
+        if keep_ticks>=0:
+            if p >= nplot - ncol or keep_ticks:
+                # 最底行画横坐标
+                ax.set_xticks(xticks)
+                ax.set_xticklabels(xticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
+            else:
+                ax.set_xticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
 
-        ax.set_xticks(xticks)
-        ax.set_xticklabels(xticks_label, fontsize=sup_fontsize * 0.9, family='Times New Roman')
-        ax.set_yticks(yticks)
-        ax.set_yticklabels(yticks_label, fontsize=sup_fontsize * 0.9, family='Times New Roman')
 
+            if pi ==0 or keep_ticks:
+                # 最左侧画纵坐标
+                ax.set_yticks(yticks)
+                ax.set_yticklabels(yticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
+            else:
+                ax.set_yticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
+        else:
+            ax.set_xticks([])
+            ax.set_yticks([])
         if grid: plt.grid(linestyle = "--",linewidth = 0.5)
         if title_list is None:
             sub_title = None
@@ -238,7 +255,7 @@ def creat_axs(nplot,map_extend,ncol = None,height  = None,width = None,dpi = 300
         add_map(ax,add_county_line=add_county_line,add_worldmap=add_worldmap,sup_fontsize=sup_fontsize,title=sub_title,
                 linewidth = linewidth,color =color)
 
-        if len(ax_index)>1:
+        if len(ax_index)>=1:
             ix = slon + 0.02*(elon - slon) * 5 / width_map
             iy = elat - 0.035*(elat - slat) * 5 / height_map
             plt.text(ix, iy, ax_index[p], bbox=dict(fc='white', ec='white',pad = 0),fontsize = sup_fontsize,zorder=100)
@@ -283,12 +300,14 @@ def creat_axs(nplot,map_extend,ncol = None,height  = None,width = None,dpi = 300
         return ax_list,min_ax_list
 
 
-def add_contourf(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,cut_colorbar = True,title = None,title_fontsize = 8,clip = None):
+
+def add_contourf(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,cut_colorbar = True,title = None,title_fontsize = 8,clip = None,
+                 extend = None,colorbar_location = None,alpha=1):
     slon = ax.transLimits._boxin.x0
     elon = ax.transLimits._boxin.x1
     slat = ax.transLimits._boxin.y0
     elat = ax.transLimits._boxin.y1
-
+    if grd is None:return
     grid0 = meteva.base.get_grid_of_data(grd)
     grid1 = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat])
     grd1 = meteva.base.interp_gg_linear(grd,grid1,outer_value=np.nan)
@@ -297,17 +316,27 @@ def add_contourf(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,cut_colo
     y = grd1['lat'].values
     vmax = np.nanmax(grd1.values)
     vmin = np.nanmin(grd1.values)
-
-    cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax,cut_colorbar = cut_colorbar)
-    norm = BoundaryNorm(clevs1, ncolors=cmap1.N-1)
-    im = ax.contourf(x, y, np.squeeze(grd1.values), levels=clevs1, cmap=cmap1,norm = norm)
-
+    if extend is None: extend = "neither"
+    cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax,cut_colorbar = cut_colorbar,extend =extend)
+    # if extend == "neither":
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N - 1)
+    # elif extend == "both":
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N + 1)
+    # else:
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
+    norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
+    im = ax.contourf(x, y, np.squeeze(grd1.values), levels=clevs1, cmap=cmap1,norm = norm,alpha=alpha,extend = extend)
     fig = plt.gcf()
     width = fig.bbox.width/fig.dpi
     height = fig.bbox.height/fig.dpi
-    location = [ax.bbox.x1/fig.dpi/width+0.005, ax.bbox.y0 / fig.dpi/height, 0.01, ax.bbox.height/fig.dpi/height]
+
     ax.set_title(title,fontsize =title_fontsize)
     if add_colorbar:
+        if colorbar_location is None:
+            location = [ax.bbox.x1 / fig.dpi / width + 0.005, ax.bbox.y0 / fig.dpi / height, 0.01,
+                        ax.bbox.height / fig.dpi / height]
+        else:
+            location = colorbar_location
         colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
         plt.colorbar(im,cax= colorbar_position)
 
@@ -393,7 +422,7 @@ def add_contour(ax,grd,color='k', linewidth = 1,label_fontsize = 5,clevs = None,
 
 
 
-def add_mesh(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,title = None,title_fontsize = 8):
+def add_mesh(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,title = None,title_fontsize = 8,extend = None,colorbar_location = None):
 
 
     slon = ax.transLimits._boxin.x0
@@ -409,23 +438,39 @@ def add_mesh(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,title = None
     y = grd1['lat'].values
     vmax = np.nanmax(grd1.values)
     vmin = np.nanmin(grd1.values)
-    cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax)
-    norm = BoundaryNorm(clevs1, ncolors=cmap1.N-1)
+    if extend is None: extend="neither"
+    cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax,extend=extend)
+    # if extend is None:
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N - 1)
+    # elif extend == "both":
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
+    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N + 1, extend=extend)
+    # else:
+    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N,extend = extend)
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
+    norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
     im = ax.pcolormesh(x, y, np.squeeze(grd1.values), cmap=cmap1, norm=norm)
     #im = ax.contourf(x, y, np.squeeze(grd.values), levels=clevs1, cmap=cmap1,norm = norm)
 
     fig = plt.gcf()
-    width = fig.bbox.width/fig.dpi
-    height = fig.bbox.height/fig.dpi
-    left_low = (width +0.1 - 0.8) / width
-    location = [ax.bbox.x1/fig.dpi/width+0.005, ax.bbox.y0 / fig.dpi/height, 0.01, ax.bbox.height/fig.dpi/height]
+
+    #left_low = (width +0.1 - 0.8) / width
+
     ax.set_title(title,fontsize =title_fontsize)
     if add_colorbar:
+        if colorbar_location is None:
+            width = fig.bbox.width / fig.dpi
+            height = fig.bbox.height / fig.dpi
+            location = [ax.bbox.x1 / fig.dpi / width + 0.005, ax.bbox.y0 / fig.dpi / height, 0.01,
+                        ax.bbox.height / fig.dpi / height]
+        else:
+            location = colorbar_location
+
         colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
-        plt.colorbar(im,cax= colorbar_position)
+        plt.colorbar(im,cax= colorbar_position,extend = extend)
     return im
 
-def add_barbs(ax,wind,color = "k",skip = None,title = None,title_fontsize = 8):
+def add_barbs(ax,wind,color = "k",skip = None,title = None,title_fontsize = 8,length = None):
 
     slon = ax.transLimits._boxin.x0
     elon = ax.transLimits._boxin.x1
@@ -446,9 +491,11 @@ def add_barbs(ax,wind,color = "k",skip = None,title = None,title_fontsize = 8):
     width = fig.bbox.width/fig.dpi
 
     if skip is  not None:
-        length = math.sqrt((width-1) * skip / x.size) * 8
+        if length is None:
+            length = math.sqrt((width-1) * skip / x.size) * 8
     else:
-        length = ax.bbox.width/fig.dpi
+        if length is None:
+            length = ax.bbox.width/fig.dpi
         skip =int((length /10)**2 * x.size / (width - 1))+1
     ax.set_title(title,fontsize =title_fontsize)
     im = ax.barbs(X[::skip,::skip],Y[::skip,::skip] , u[::skip,::skip], v[::skip,::skip],
@@ -460,7 +507,7 @@ def add_barbs(ax,wind,color = "k",skip = None,title = None,title_fontsize = 8):
 
 
 def add_scatter(ax,sta0,cmap = "rainbow",clevs = None,point_size = None,fix_size = True,threshold = 2,min_spot_value = 0,mean_value = 2,
-                add_colorbar = True,alpha = None,title = None,title_fontsize = 8):
+                add_colorbar = True,alpha = None,title = None,title_fontsize = 8,extend = None,colorbar_location = None):
 
     slon = ax.transLimits._boxin.x0
     elon = ax.transLimits._boxin.x1
@@ -478,10 +525,17 @@ def add_scatter(ax,sta0,cmap = "rainbow",clevs = None,point_size = None,fix_size
 
     vmax_v = np.max(sta_without_iv.iloc[:,-1].values)
     vmin_v = np.min(sta_without_iv.iloc[:,-1].values)
-
-    cmap1, clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap, clevs=clevs, vmin=vmin_v, vmax=vmax_v)
-
-    norm = BoundaryNorm(clevs1, ncolors=cmap1.N - 1)
+    if extend is None: extend="neither"
+    cmap1, clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap, clevs=clevs, vmin=vmin_v, vmax=vmax_v,extend = extend)
+    # if extend is None:
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N - 1)
+    # elif extend == "both":
+    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N + 2, extend=extend)
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
+    # else:
+    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N+1, extend=extend)
+    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
+    norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
     fig = plt.gcf()
     map_width = ax.bbox.width/fig.dpi
 
@@ -522,14 +576,16 @@ def add_scatter(ax,sta0,cmap = "rainbow",clevs = None,point_size = None,fix_size
 
 
     if add_colorbar:
-        width = fig.bbox.width / fig.dpi
-        height = fig.bbox.height / fig.dpi
-        location = [ax.bbox.x1 / fig.dpi / width + 0.005, ax.bbox.y0 / fig.dpi / height, 0.01,
-                    ax.bbox.height / fig.dpi / height]
+        if colorbar_location is None:
+            width = fig.bbox.width / fig.dpi
+            height = fig.bbox.height / fig.dpi
+            location = [ax.bbox.x1 / fig.dpi / width + 0.005, ax.bbox.y0 / fig.dpi / height, 0.01,
+                        ax.bbox.height / fig.dpi / height]
+        else:
+            location = colorbar_location
 
-        if (add_colorbar):
-            colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
-            plt.colorbar(im, cax=colorbar_position)
+        colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
+        plt.colorbar(im, cax=colorbar_position,extend = extend)
     return im
 
 def add_scatter_text(ax,sta0,color = "k",cmap = None,clevs = None,tag = 2,
@@ -549,7 +605,6 @@ def add_scatter_text(ax,sta0,color = "k",cmap = None,clevs = None,tag = 2,
     cmap1 = None
     if cmap is not None:
         cmap1, clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap, clevs=clevs, vmin=vmin_v, vmax=vmax_v)
-
     fig = plt.gcf()
     fmt_tag = "%." + str(tag) + "f"
 
@@ -568,7 +623,7 @@ def add_scatter_text(ax,sta0,color = "k",cmap = None,clevs = None,tag = 2,
         y = sta_without_iv.iloc[:, 5].values
         v = sta_without_iv.iloc[:, -1].values
         for k in range(len(clevs1)-1):
-            index = np.where((v>clevs1[k])&(v<clevs1[k+1]))
+            index = np.where((v>=clevs1[k])&(v<clevs1[k+1]))
             if len(index[0])>0:
                 x1 = x[index]
                 y1 = y[index]
@@ -584,6 +639,8 @@ def add_scatter_text(ax,sta0,color = "k",cmap = None,clevs = None,tag = 2,
     return
 
 def add_closed_line(ax,graphy,color = "k",linewidth=2,fontsize = 10,title = None,title_fontsize = 8):
+    if graphy is None:
+        return
     x0 = ax.transLimits._boxin.x0
     x1 = ax.transLimits._boxin.x1
     y0 = ax.transLimits._boxin.y0
@@ -642,8 +699,9 @@ def add_closed_line(ax,graphy,color = "k",linewidth=2,fontsize = 10,title = None
     return
 
 
-
 def add_shear_line(ax,graphy,linewidth = 1,title = None,title_fontsize = 8):
+    if graphy is None:
+        return
     slon = ax.transLimits._boxin.x0
     elon = ax.transLimits._boxin.x1
     slat = ax.transLimits._boxin.y0
@@ -680,7 +738,9 @@ def add_shear_line(ax,graphy,linewidth = 1,title = None,title_fontsize = 8):
     ax.set_title(title,fontsize =title_fontsize)
     return
 
-def add_trough_axes(ax,graphy,linewidths = None,title = None,title_fontsize = 8):
+def add_trough_axes(ax,graphy,color = "r",linewidths = None,title = None,title_fontsize = 8):
+    if graphy is None:
+        return
     slon = ax.transLimits._boxin.x0
     elon = ax.transLimits._boxin.x1
     slat = ax.transLimits._boxin.y0
@@ -698,14 +758,17 @@ def add_trough_axes(ax,graphy,linewidths = None,title = None,title_fontsize = 8)
     for value in features.values():
         line = value["axes"]
         point = np.array(line["point"])
-        ax.plot(point[:, 0], point[:, 1], "r", linewidth=linewidths)
+        if point.size>2:
+            ax.plot(point[:, 0], point[:, 1],color, linewidth=linewidths)
 
     ax.set_xlim(slon,elon)
     ax.set_ylim(slat,elat)
     ax.set_title(title,fontsize =title_fontsize)
     return
 
-def add_jet_axes(ax,graphy,title = None,title_fontsize = 8):
+def add_jet_axes(ax,graphy,title = None,title_fontsize = 8,color ="yellow" ):
+    if graphy is None:
+        return
     slon = ax.transLimits._boxin.x0
     elon = ax.transLimits._boxin.x1
     slat = ax.transLimits._boxin.y0
@@ -723,27 +786,27 @@ def add_jet_axes(ax,graphy,title = None,title_fontsize = 8):
         line = value["axes"]
         point = np.array(line["point"])
         npoint = len(line["point"])
+        if npoint>2:
+            ns = npoint -1
+            dx = 0
+            dy = 0
+            while ns >0:
+                ns -= 1
+                dx = point[npoint-1,0] - point[ns,0]
+                dy = point[npoint-1,1] - point[ns,1]
+                dis = (dx**2 + dy**2)**0.5
+                if dis > 0.3:
+                    break
 
-        ns = npoint -1
-        dx = 0
-        dy = 0
-        while ns >0:
-            ns -= 1
-            dx = point[npoint-1,0] - point[ns,0]
-            dy = point[npoint-1,1] - point[ns,1]
-            dis = (dx**2 + dy**2)**0.5
-            if dis > 0.3:
-                break
-
-        ax.arrow(point[ns,0],point[ns,1],dx*0.01,dy*0.01,head_width=0.3,head_length = 0.3,fc = "yellow",ec = "yellow")
-        ax.plot(point[:ns, 0], point[:ns, 1], "yellow", linewidth=2)
+            ax.arrow(point[ns-1,0],point[ns-1,1],dx*0.01,dy*0.01,head_width=1,head_length = 1,fc = color,ec = color)
+            ax.plot(point[:ns, 0], point[:ns, 1], color, linewidth=2.5)
 
     ax.set_xlim(slon,elon)
     ax.set_ylim(slat,elat)
     ax.set_title(title,fontsize =title_fontsize)
     return
 
-def add_cyclone_trace(ax,sta_cyclone_trace,size = 0.2,linewidth = 1,title = None,title_fontsize = 8):
+def add_cyclone_trace(ax,sta_cyclone_trace,size = 0.3,linewidth = 1,title = None,title_fontsize = 8):
 
     np = len(sta_cyclone_trace.index)
     for i in range(np - 1):
@@ -752,18 +815,18 @@ def add_cyclone_trace(ax,sta_cyclone_trace,size = 0.2,linewidth = 1,title = None
         speed = sta_cyclone_trace.iloc[i+1,-1]
 
 
-        if speed > 10.8 and speed <17.2:
-            color = "yellow"
+        if speed >= 10.8 and speed <17.2:
+            color = "k"
         elif speed >=17.2 and speed <24.5:
             color = "b"
         elif speed >= 24.5 and speed < 32.7:
-            color = "g"
+            color = "y"
         elif speed >= 32.7 and speed < 41.5:
             color = "orange"
         elif speed >=41.5 and speed < 51:
-            color = "m"
-        else:
             color = "r"
+        else:
+            color = "m"
         ax.plot(lon, lat, c = color, linewidth=linewidth, zorder=29)
         time_ob = sta_cyclone_trace.iloc[i,1]
 
@@ -784,7 +847,7 @@ def add_cyclone_trace(ax,sta_cyclone_trace,size = 0.2,linewidth = 1,title = None
                     lat1 = lat[0] + r_o * math.sin(theta)
                 lon_list.append(lon1)
                 lat_list.append(lat1)
-            ax.fill(lon_list,lat_list,c = color,zorder = 30)
+            ax.fill(lon_list,lat_list,c = color,zorder = 30,linewidth=0)
             r_o = size * 0.3
             lon_list = []
             lat_list = []
@@ -794,13 +857,14 @@ def add_cyclone_trace(ax,sta_cyclone_trace,size = 0.2,linewidth = 1,title = None
                 lat1 = lat[0] + r_o * math.sin(theta)
                 lon_list.append(lon1)
                 lat_list.append(lat1)
-            ax.fill(lon_list,lat_list,c = "white",zorder = 30)
+            ax.fill(lon_list,lat_list,c = "white",zorder = 30,linewidth=0)
     ax.set_title(title,fontsize =title_fontsize)
     return
 
 
 def add_lines(ax,graphy,cent = None,color = "k",linestyle = "-",linewidths = None,title = None,title_fontsize = 8):
-
+    if graphy is None:
+        return
     slon = ax.transLimits._boxin.x0
     elon = ax.transLimits._boxin.x1
     slat = ax.transLimits._boxin.y0
@@ -832,11 +896,12 @@ def add_lines(ax,graphy,cent = None,color = "k",linestyle = "-",linewidths = Non
                 level = df["level"].values[0]
                 time = df["time"].values[0]
                 dtime = df["dtime"].values[0]
-                cent1 = meb.sele_by_para(cent,level = level,time = time,dtime = dtime,id = id)
+                cent1 = meteva.base.sele_by_para(cent,level = level,time = time,dtime = dtime,id = id)
 
 
 
     ax.set_xlim(slon,elon)
     ax.set_ylim(slat,elat)
     ax.set_title(title,fontsize =title_fontsize)
-    return
\ No newline at end of file
+    return
+
