@@ -1,7 +1,7 @@
 from . import score
 from . import plot
 
-from .plot import  scatter_uv,scatter_uv_error,statisitic_uv
+from .plot import  scatter_uv,scatter_uv_error,statisitic_uv,probability_density_uv,statistic_uv
 from .score import acd_nas,acs_nasws,acz_na,acs,acz,nas_d,nasws_s,na_ds,scd_nas,scd,scs,scs_nasws,acd
 from .score import wind_weaker_rate,wind_severer_rate,wind_severer_rate_nasws,wind_weaker_rate_nasws
 from .score import nasws_uv,na_uv,acd_uv,acs_uv,acz_uv,nas_uv,scd_uv,scs_uv,wind_severer_rate_uv,wind_weaker_rate_uv
