@@ -8,8 +8,11 @@ from .table import *
 from .plot import scatter_regress,pdf_plot,box_plot_continue,taylor_diagram,frequency_histogram_error,accumulation_change_with_strenght,frequency_change_with_strenght
 from .skill import mre_skill,sme,sst
 from .score import sample_count,ob_mean,fo_mean,tase,me,mae,mse,rmse,me_tase,mae_tase,mse_tase,rmse_tase,tc_count
-from .score import bias_m,bias_tmmsss,corr,corr_tmmsss,tmmsss,tmmsss_merge,toar,nse,nse_tase_tmmsss,mre,mre_toar,correct_rate,correct_rate_tc
+from .score import bias_m,bias_tmmsss,corr,corr_tmmsss,tmmsss,tmmsss_merge,toar,nse,nse_tase_tmmsss,nse_tmmsss,mre,mre_toar,correct_rate,correct_rate_tc
 from .score import ob_fo_mean,ob_fo_std,ob_fo_max,ob_fo_min,ob_fo_cv
 from .score import wrong_rate,wrong_rate_tc,residual_error,residual_error_tmmsss,residual_error_rate,residual_error_rate_tmmsss
 from .score import max_abs_error,max_error,min_error
-from .score import ob_fo_sum,rmsf,tlfo,corr_rank,rmsf_tlfo,ob_fo_quantile,ob_fo_precipitation_strenght
\ No newline at end of file
+from .score import ob_fo_sum,rmsf,tlfo,corr_rank,rmsf_tlfo,ob_fo_quantile
+from .score import ob_fo_sum_tmmsss,ob_fo_mean_tmmsss,ob_fo_std_tmmsss
+from .score import pas,pmse
+from .score import ob_fo_precipitation_strength,ob_fo_precipitation_strength_cscs,cscs,ob_fo_precipitation_strenght
\ No newline at end of file
