@@ -269,6 +269,12 @@ def ob_fo_std(ob,fo):
     return result
 
 def ob_fo_cv(ob,fo):
+    '''
+    观测和预报的离差
+    :param ob:
+    :param fo:
+    :return:
+    '''
     Fo_shape = fo.shape
     Ob_shape = ob.shape
 
@@ -290,8 +296,74 @@ def ob_fo_cv(ob,fo):
     return result
 
 
+def cscs(Ob,Fo):
+    '''
+       :param ob: 观测降水序列
+       :param fo: 预报降水序列
+       :return: 观测和预报各自的平均降水强度计算相关的中间量
+       '''
+    cscs_list = []
+    Fo_shape = Fo.shape
+    Ob_shape = Ob.shape
 
-def ob_fo_precipitation_strenght(ob,fo):
+    Ob_shpe_list = list(Ob_shape)
+    size = len(Ob_shpe_list)
+    ind = -size
+    Fo_Ob_index = list(Fo_shape[ind:])
+    if Fo_Ob_index != Ob_shpe_list:
+        print('预报数据和观测数据维度不匹配')
+        return
+    Ob_shpe_list.insert(0, -1)
+    new_Fo_shape = tuple(Ob_shpe_list)
+    new_Fo = Fo.reshape(new_Fo_shape)
+    new_Fo_shape = new_Fo.shape
+
+    ob_not_0 = Ob[Ob >= 0.1]
+    cob = ob_not_0.size
+    sob = np.sum(ob_not_0)
+
+    for line in range(new_Fo_shape[0]):
+        foi = new_Fo[line, :]
+        fo_not_0 = foi[foi >= 0.1]
+        cfo = fo_not_0.size
+        sfo = np.sum(fo_not_0)
+        cscs_list.append(np.array([cob,sob, cfo,sfo]))
+
+    cscs_np = np.array(cscs_list)
+    shape = list(Fo_shape[:ind])
+    shape.append(4)
+
+    cscs_array =cscs_np.reshape(shape)
+    return cscs_array
+
+def ob_fo_precipitation_strength_cscs(cscs_array):
+
+    if len(cscs_array.shape) == 1:
+        cscs_array = cscs_array.reshape(1,1, 4)
+    cob = cscs_array[..., 0]
+    sob = cscs_array[..., 1]
+    cfo = cscs_array[..., 2]
+    sfo = cscs_array[..., 3]
+
+    sob[cob==0] = IV
+    cob[cob == 0] = 1
+    sfo[cfo == 0] =IV
+    cfo[cfo == 0] = 1
+
+    shape1 = list(cscs_array.shape)
+    if len(cscs_array.shape) == 2:
+        result = np.zeros((2, shape1[0]))
+        result[0, :] = sob / cob
+        result[1, :] = sfo / cfo
+    else:
+        result = np.zeros((1 + shape1[0], shape1[1]))
+        result[0, :] = sob / cob
+        result[1, :] = sfo / cfo
+
+    return result
+
+
+def ob_fo_precipitation_strength(ob,fo):
     '''
     :param ob: 观测降水序列
     :param fo: 预报降水序列
@@ -322,6 +394,11 @@ def ob_fo_precipitation_strenght(ob,fo):
     return result
 
 
+def ob_fo_precipitation_strenght(ob,fo):
+    result = ob_fo_precipitation_strength(ob,fo)
+    return result
+
+
 
 def ob_mean(Ob, Fo=None):
     '''
@@ -526,7 +603,7 @@ def rmsf_tlfo(tlfo_array):
     return rmsf
 
 
-def tase(Ob, Fo):
+def tase(Ob, Fo,weight = None):
     '''
     计算平均误差、平均绝对误差、均方误差、均方根误差的中间结果
     -----------------------------
@@ -538,7 +615,6 @@ def tase(Ob, Fo):
     tase_list = []
     Fo_shape = Fo.shape
     Ob_shape = Ob.shape
-
     Ob_shpe_list = list(Ob_shape)
     size = len(Ob_shpe_list)
     ind = -size
@@ -550,19 +626,28 @@ def tase(Ob, Fo):
     new_Fo_shape = tuple(Ob_shpe_list)
     new_Fo = Fo.reshape(new_Fo_shape)
     new_Fo_shape = new_Fo.shape
-    for line in range(new_Fo_shape[0]):
-        total_count = Ob.size
-        e_sum = np.sum(new_Fo[line, :] - Ob)
-        ae_sum = np.sum(np.abs(new_Fo[line, :] - Ob))
-        se_sum = np.sum(np.square(new_Fo[line, :] - Ob))
-        tase_list.append(np.array([total_count, e_sum, ae_sum, se_sum]))
+    if weight is None:
+        for line in range(new_Fo_shape[0]):
+            total_count = Ob.size
+            e_sum = np.sum(new_Fo[line, :] - Ob)
+            ae_sum = np.sum(np.abs(new_Fo[line, :] - Ob))
+            se_sum = np.sum(np.square(new_Fo[line, :] - Ob))
+            tase_list.append(np.array([total_count, e_sum, ae_sum, se_sum]))
+    else:
+        for line in range(new_Fo_shape[0]):
+            total_count = np.sum(weight)
+            e_sum = np.sum((new_Fo[line, :] - Ob) * weight)
+            ae_sum = np.sum(np.abs(new_Fo[line, :] - Ob) * weight)
+            se_sum = np.sum(np.square(new_Fo[line, :] - Ob)*weight)
+            tase_list.append(np.array([total_count, e_sum, ae_sum, se_sum]))
+
     tase_np = np.array(tase_list)
     shape = list(Fo_shape[:ind])
     shape.append(4)
-
     tase_array = tase_np.reshape(shape)
     return tase_array
 
+
 def max_error(Ob,Fo):
     me_list = []
     Fo_shape = Fo.shape
@@ -645,7 +730,8 @@ def max_abs_error(Ob,Fo):
         error_array  = np.max(np.abs(Fo - Ob))
     return error_array
 
-def me(Ob, Fo):
+
+def me(Ob, Fo,weight = None):
     '''
     me 求两组数据的误差平均值
     -----------------------------
@@ -653,30 +739,36 @@ def me(Ob, Fo):
     :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
     :return: 负无穷到正无穷的实数，最优值为0
     '''
-    me_list = []
-    Fo_shape = Fo.shape
-    Ob_shape = Ob.shape
 
-    Ob_shpe_list = list(Ob_shape)
-    size = len(Ob_shpe_list)
-    ind = -size
-    Fo_Ob_index = list(Fo_shape[ind:])
-    if Fo_Ob_index != Ob_shpe_list:
-        print('预报数据和观测数据维度不匹配')
-        return
-    if len(Fo_shape)> len(Ob_shape):
-        Ob_shpe_list.insert(0, -1)
-        new_Fo_shape = tuple(Ob_shpe_list)
-        new_Fo = Fo.reshape(new_Fo_shape)
-        new_Fo_shape = new_Fo.shape
-        for line in range(new_Fo_shape[0]):
-            mean_error = np.mean(new_Fo[line, :] - Ob)
-            me_list.append(mean_error)
-        mean_error_array = np.array(me_list)
-        shape = list(Fo_shape[:ind])
-        mean_error_array = mean_error_array.reshape(shape)
+    if weight is None:
+        me_list = []
+        Fo_shape = Fo.shape
+        Ob_shape = Ob.shape
+
+        Ob_shpe_list = list(Ob_shape)
+        size = len(Ob_shpe_list)
+        ind = -size
+        Fo_Ob_index = list(Fo_shape[ind:])
+        if Fo_Ob_index != Ob_shpe_list:
+            print('预报数据和观测数据维度不匹配')
+            return
+        if len(Fo_shape)> len(Ob_shape):
+            Ob_shpe_list.insert(0, -1)
+            new_Fo_shape = tuple(Ob_shpe_list)
+            new_Fo = Fo.reshape(new_Fo_shape)
+            new_Fo_shape = new_Fo.shape
+            for line in range(new_Fo_shape[0]):
+                mean_error = np.mean(new_Fo[line, :] - Ob)
+                me_list.append(mean_error)
+            mean_error_array = np.array(me_list)
+            shape = list(Fo_shape[:ind])
+            mean_error_array = mean_error_array.reshape(shape)
+        else:
+            mean_error_array  = np.mean(Fo - Ob)
     else:
-        mean_error_array  = np.mean(Fo - Ob)
+        tase_array = tase(Ob,Fo,weight)
+        mean_error_array = me_tase(tase_array)
+
     return mean_error_array
 
 
@@ -691,7 +783,7 @@ def me_tase(tase_array):
     return mean_error
 
 
-def mae(Ob, Fo):
+def mae(Ob, Fo,weight = None):
     '''
     me 求两组数据的误差平均值
     -----------------------------
@@ -699,31 +791,37 @@ def mae(Ob, Fo):
     :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
     :return: 负无穷到正无穷的实数，最优值为0
     '''
-    mae_list = []
-    Fo_shape = Fo.shape
-    Ob_shape = Ob.shape
+    if weight is None:
+        mae_list = []
+        Fo_shape = Fo.shape
+        Ob_shape = Ob.shape
 
-    Ob_shpe_list = list(Ob_shape)
-    size = len(Ob_shpe_list)
-    ind = -size
-    Fo_Ob_index = list(Fo_shape[ind:])
-    if Fo_Ob_index != Ob_shpe_list:
-        print('预报数据和观测数据维度不匹配')
-        return
-    if len(Fo_shape) == len(Ob_shape):
-        mean_abs_error = np.mean(np.abs(Fo - Ob))
-        return mean_abs_error
+        Ob_shpe_list = list(Ob_shape)
+        size = len(Ob_shpe_list)
+        ind = -size
+        Fo_Ob_index = list(Fo_shape[ind:])
+        if Fo_Ob_index != Ob_shpe_list:
+            print('预报数据和观测数据维度不匹配')
+            return
+        if len(Fo_shape) == len(Ob_shape):
+            mean_abs_error = np.mean(np.abs(Fo - Ob))
+            return mean_abs_error
+        else:
+            Ob_shpe_list.insert(0, -1)
+            new_Fo_shape = tuple(Ob_shpe_list)
+            new_Fo = Fo.reshape(new_Fo_shape)
+            new_Fo_shape = new_Fo.shape
+            for line in range(new_Fo_shape[0]):
+                mean_abs_error = np.mean(np.abs(new_Fo[line, :] - Ob))
+                mae_list.append(mean_abs_error)
+            mean_error_array = np.array(mae_list)
+            shape = list(Fo_shape[:ind])
+            mean_abs_error_array = mean_error_array.reshape(shape)
+            return mean_abs_error_array
     else:
-        Ob_shpe_list.insert(0, -1)
-        new_Fo_shape = tuple(Ob_shpe_list)
-        new_Fo = Fo.reshape(new_Fo_shape)
-        new_Fo_shape = new_Fo.shape
-        for line in range(new_Fo_shape[0]):
-            mean_abs_error = np.mean(np.abs(new_Fo[line, :] - Ob))
-            mae_list.append(mean_abs_error)
-        mean_error_array = np.array(mae_list)
-        shape = list(Fo_shape[:ind])
-        mean_abs_error_array = mean_error_array.reshape(shape)
+        tase_array = tase(Ob, Fo, weight)
+        mean_abs_error_array = mae_tase(tase_array)
+
         return mean_abs_error_array
 
 
@@ -738,7 +836,7 @@ def mae_tase(tase_array):
     return mean_abs_error
 
 
-def mse(Ob, Fo):
+def mse(Ob, Fo,weight = None):
     '''
     mean_sqrt_error, 求两组数据的均方误差
     ----------------------------------
@@ -746,34 +844,38 @@ def mse(Ob, Fo):
     :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
     :return: 0到无穷大，最优值为0
     '''
+    if weight is None:
+        mse_list = []
+        Fo_shape = Fo.shape
+        Ob_shape = Ob.shape
 
-    mse_list = []
-    Fo_shape = Fo.shape
-    Ob_shape = Ob.shape
-
-    Ob_shpe_list = list(Ob_shape)
-    size = len(Ob_shpe_list)
-    ind = -size
-    Fo_Ob_index = list(Fo_shape[ind:])
-    if Fo_Ob_index != Ob_shpe_list:
-        print('预报数据和观测数据维度不匹配')
-        return
-    if len(Fo_shape) == len(Ob_shape):
-        mean_square_error = np.mean(np.square(Fo - Ob))
-        return mean_square_error
+        Ob_shpe_list = list(Ob_shape)
+        size = len(Ob_shpe_list)
+        ind = -size
+        Fo_Ob_index = list(Fo_shape[ind:])
+        if Fo_Ob_index != Ob_shpe_list:
+            print('预报数据和观测数据维度不匹配')
+            return
+        if len(Fo_shape) == len(Ob_shape):
+            mean_square_error = np.mean(np.square(Fo - Ob))
+            return mean_square_error
+        else:
+            Ob_shpe_list.insert(0, -1)
+            new_Fo_shape = tuple(Ob_shpe_list)
+            new_Fo = Fo.reshape(new_Fo_shape)
+            new_Fo_shape = new_Fo.shape
+            for line in range(new_Fo_shape[0]):
+                mean_square_error = np.mean(np.square(new_Fo[line, :] - Ob))
+                mse_list.append(mean_square_error)
+            mean_sqrt_array = np.array(mse_list)
+            shape = list(Fo_shape[:ind])
+            mean_sqrt_error_array = mean_sqrt_array.reshape(shape)
+            return mean_sqrt_error_array
     else:
-        Ob_shpe_list.insert(0, -1)
-        new_Fo_shape = tuple(Ob_shpe_list)
-        new_Fo = Fo.reshape(new_Fo_shape)
-        new_Fo_shape = new_Fo.shape
-        for line in range(new_Fo_shape[0]):
-            mean_square_error = np.mean(np.square(new_Fo[line, :] - Ob))
-            mse_list.append(mean_square_error)
-        mean_sqrt_array = np.array(mse_list)
-        shape = list(Fo_shape[:ind])
-        mean_sqrt_error_array = mean_sqrt_array.reshape(shape)
-        return mean_sqrt_error_array
+        tase_array = tase(Ob, Fo, weight)
+        mean_squre_error = mse_tase(tase_array)
 
+        return mean_squre_error
 
 def mse_tase(tase_array):
     '''
@@ -786,7 +888,7 @@ def mse_tase(tase_array):
     return mean_squre_error
 
 
-def rmse(Ob, Fo):
+def rmse(Ob, Fo,weight = None):
     '''
     root_mean_square_error 求两组数据的均方根误差
     ------------------------------
@@ -794,33 +896,38 @@ def rmse(Ob, Fo):
     :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
     :return: 0到无穷大，最优值为0
     '''
-    rmse_list = []
-    Fo_shape = Fo.shape
-    Ob_shape = Ob.shape
+    if weight is None:
+        rmse_list = []
+        Fo_shape = Fo.shape
+        Ob_shape = Ob.shape
 
-    Ob_shpe_list = list(Ob_shape)
-    size = len(Ob_shpe_list)
-    ind = -size
-    Fo_Ob_index = list(Fo_shape[ind:])
-    if Fo_Ob_index != Ob_shpe_list:
-        print('预报数据和观测数据维度不匹配')
-        return
-    if len(Fo_shape) == len(Ob_shape):
-        mean_square_error = np.sqrt(np.mean(np.square(Fo - Ob)))
-        return mean_square_error
+        Ob_shpe_list = list(Ob_shape)
+        size = len(Ob_shpe_list)
+        ind = -size
+        Fo_Ob_index = list(Fo_shape[ind:])
+        if Fo_Ob_index != Ob_shpe_list:
+            print('预报数据和观测数据维度不匹配')
+            return
+        if len(Fo_shape) == len(Ob_shape):
+            mean_square_error = np.sqrt(np.mean(np.square(Fo - Ob)))
+            return mean_square_error
+        else:
+            Ob_shpe_list.insert(0, -1)
+            new_Fo_shape = tuple(Ob_shpe_list)
+            new_Fo = Fo.reshape(new_Fo_shape)
+            new_Fo_shape = new_Fo.shape
+            for line in range(new_Fo_shape[0]):
+                root_mean_sqrt_error = np.sqrt(np.mean(np.square(new_Fo[line, :] - Ob)))
+                rmse_list.append(root_mean_sqrt_error)
+            root_mean_sqrt_array = np.array(rmse_list)
+            shape = list(Fo_shape[:ind])
+            root_mean_sqrt_error_array = root_mean_sqrt_array.reshape(shape)
+            return root_mean_sqrt_error_array
     else:
-        Ob_shpe_list.insert(0, -1)
-        new_Fo_shape = tuple(Ob_shpe_list)
-        new_Fo = Fo.reshape(new_Fo_shape)
-        new_Fo_shape = new_Fo.shape
-        for line in range(new_Fo_shape[0]):
-            root_mean_sqrt_error = np.sqrt(np.mean(np.square(new_Fo[line, :] - Ob)))
-            rmse_list.append(root_mean_sqrt_error)
-        root_mean_sqrt_array = np.array(rmse_list)
-        shape = list(Fo_shape[:ind])
-        root_mean_sqrt_error_array = root_mean_sqrt_array.reshape(shape)
-        return root_mean_sqrt_error_array
+        tase_array = tase(Ob, Fo, weight)
+        root_mean_sqrt_error = rmse_tase(tase_array)
 
+        return root_mean_sqrt_error
 
 def rmse_tase(tase_array):
     '''
@@ -929,7 +1036,7 @@ def bias_tmmsss(tmmsss_array):
     return bias0
 
 
-def corr(Ob, Fo):
+def corr(Ob, Fo,weight = None):
     '''
     相关系数，求实况数据还和预测数据之间的相关系数
     -----------------------------
@@ -937,7 +1044,7 @@ def corr(Ob, Fo):
     :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
     :return: corr0
     '''
-    tmmsss_array = tmmsss(Ob,Fo)
+    tmmsss_array = tmmsss(Ob,Fo,weight = weight)
     corr0 = corr_tmmsss(tmmsss_array)
     return corr0
 
@@ -1043,8 +1150,77 @@ def corr_tmmsss(tmmsss_array):
     corr = sxy / sxxsyy
     return corr
 
+def ob_fo_sum_tmmsss(tmmsss_array):
+    '''
+    相关系数，求实况数据还和预测数据之间的相关系数
+    :param tmmsss_array: 包含命中空报和漏报的多维数组，其中最后一维长度为6，分别记录了（count,mx,my,sxx,syy,sxy）
+    :return:
+    '''
+    if len(tmmsss_array.shape) == 1:
+        tmmsss_array = tmmsss_array.reshape(1, 1, 6)
+    mx = tmmsss_array[..., 1]
+    my = tmmsss_array[..., 2]
+    shape1 = list(tmmsss_array.shape)
+    if len(tmmsss_array.shape) == 2:
+        result = np.zeros((2, shape1[0]))
+        result[0, :] = mx * tmmsss_array[..., 0]
+        result[1, :] = my * tmmsss_array[..., 0]
+    else:
+        result = np.zeros((1 + shape1[0], shape1[1]))
+        result[0, :] = mx * tmmsss_array[..., 0]
+        result[1:, :] = my * tmmsss_array[..., 0]
+    return result
 
-def tmmsss(Ob, Fo):
+
+
+def ob_fo_mean_tmmsss(tmmsss_array):
+    '''
+    相关系数，求实况数据还和预测数据之间的相关系数
+    :param tmmsss_array: 包含命中空报和漏报的多维数组，其中最后一维长度为6，分别记录了（count,mx,my,sxx,syy,sxy）
+    :return:
+    '''
+
+    if len(tmmsss_array.shape) == 1:
+        tmmsss_array = tmmsss_array.reshape(1, 1, 6)
+    mx = tmmsss_array[..., 1]
+    my = tmmsss_array[..., 2]
+    shape1 = list(tmmsss_array.shape)
+    if len(tmmsss_array.shape) == 2:
+        result = np.zeros((2, shape1[0]))
+        result[0, :] = mx
+        result[1, :] = my
+    else:
+        result = np.zeros((1 + shape1[0], shape1[1]))
+        result[0, :] = mx
+        result[1:, :] = my
+    return result
+
+
+
+def ob_fo_std_tmmsss(tmmsss_array):
+    '''
+    相关系数，求实况数据还和预测数据之间的相关系数
+    :param tmmsss_array: 包含命中空报和漏报的多维数组，其中最后一维长度为6，分别记录了（count,mx,my,sxx,syy,sxy）
+    :return:
+    '''
+
+    if len(tmmsss_array.shape) == 1:
+        tmmsss_array = tmmsss_array.reshape(1, 1, 6)
+    sx = tmmsss_array[..., 3]
+    sy = tmmsss_array[..., 4]
+    shape1 = list(tmmsss_array.shape)
+    if len(tmmsss_array.shape) == 2:
+        result = np.zeros((2, shape1[0]))
+        result[0, :] = sx
+        result[1, :] = sy
+    else:
+        result = np.zeros((1 + shape1[0], shape1[1]))
+        result[0, :] = sx
+        result[1:, :] = sy
+    result = np.sqrt(result)
+    return result
+
+def tmmsss(Ob, Fo,weight = None):
     '''
     统计相关系数等检验量所需的中间变量
     :param Ob: 实况数据  任意维numpy数组
@@ -1067,18 +1243,32 @@ def tmmsss(Ob, Fo):
     new_Fo_shape = tuple(Ob_shpe_list)
     new_Fo = Fo.reshape(new_Fo_shape)
     new_Fo_shape = new_Fo.shape
-    for line in range(new_Fo_shape[0]):
-        ob_f = Ob.flatten()
-        fo_f = new_Fo[line, :].flatten()
+    ob_f = Ob.flatten()
+    if weight is None:
         count = Ob.size
         mx = np.mean(ob_f)
-        my = np.mean(fo_f)
-        dx = ob_f - mx
-        dy = fo_f - my
-        sxx = np.mean(np.power(dx, 2))
-        syy = np.mean(np.power(dy, 2))
-        sxy = np.mean(dx * dy)
-        tmmsss_array_list.append(np.array([count, mx, my, sxx, syy, sxy]))
+        for line in range(new_Fo_shape[0]):
+            fo_f = new_Fo[line, :].flatten()
+            my = np.mean(fo_f)
+            dx = ob_f - mx
+            dy = fo_f - my
+            sxx = np.mean(np.power(dx, 2))
+            syy = np.mean(np.power(dy, 2))
+            sxy = np.mean(dx * dy)
+            tmmsss_array_list.append(np.array([count, mx, my, sxx, syy, sxy]))
+    else:
+        weight_f = weight.flatten()
+        count = np.sum(weight_f)
+        mx = np.sum(ob_f * weight_f) /count
+        for line in range(new_Fo_shape[0]):
+            fo_f = new_Fo[line, :].flatten()
+            my = np.sum(fo_f * weight_f) /count
+            dx = ob_f - mx
+            dy = fo_f - my
+            sxx = np.sum(np.power(dx, 2) * weight_f)/count
+            syy = np.sum(np.power(dy, 2) * weight_f)/count
+            sxy = np.sum(dx * dy * weight_f) / count
+            tmmsss_array_list.append(np.array([count, mx, my, sxx, syy, sxy]))
     tmmsss_array = np.array(tmmsss_array_list)
     shape = list(Fo_shape[:ind])
     shape.append(6)
@@ -1093,9 +1283,9 @@ def tmmsss_merge(tmmsss0, tmmsss1):
     :param tmmsss1: 长度6的一维数组，分别记录了（count,mx,my,sxx,syy,sxy）
     :return: 长度6的一维数组，分别记录了（count,mx,my,sxx,syy,sxy）
     '''
-    if np.isnan(tmmsss0[0]):
+    if np.isnan(tmmsss0[0]).any():
         return tmmsss1
-    elif np.isnan(tmmsss1[0]):
+    elif np.isnan(tmmsss1[0]).any():
         return tmmsss0
     tmmsss_array_list = []
     tmmsss0_shape = list(tmmsss0.shape)
@@ -1306,3 +1496,208 @@ def nse_tase_tmmsss(tase_array, tmmsss_array):
         nse0[sxx == 0] = IV
     return nse0
 
+def nse_tmmsss(tmmsss_array):
+    '''
+    :param tase_array:
+    :param tmmsss_array:
+    :return:
+    '''
+
+    sxx = tmmsss_array[..., 3] + 0
+    syy = tmmsss_array[..., 4] + 0
+    sxy = tmmsss_array[..., 5] + 0
+    mx = tmmsss_array[..., 1] + 0
+    my = tmmsss_array[..., 2] + 0
+    exy = sxx + syy - 2 * sxy + (mx - my) * (mx - my)
+
+
+
+    if sxx.size == 1:
+        if sxx == 0:
+            nse0 = IV
+        else:
+            nse0 = 1 - exy / sxx
+    else:
+        sum = sxx + 0
+        sum[sxx == 0] = 1e-10
+        mse0 = exy
+        nse0 = 1 - mse0 / sum
+        nse0[sxx == 0] = IV
+    return nse0
+
+
+
+def pmse(ob,fo):
+    '''
+
+    :param ob: 一维numpy数组
+    :param fo:  一维或者两维numpy数组，当fo是两维时，表示有多种预报进行对比
+    :return:  实数
+    '''
+    #如果fo只包含一个预报成员
+    if len(ob.shape) == len(fo.shape):
+        fo = fo[np.newaxis,:]
+    nfo = fo.shape[0]
+    list1 = []
+    result_list = []
+    nsta = ob.size
+    #print("---")
+    #print(nsta)
+    #对多个预报进行循环
+    for i in range(nfo):
+        foi = fo[i,:]
+        #ob3 = np.power(ob,0.33333)
+        #fo3 = np.power(foi,0.333333)
+        e = ob - foi
+        e1 = e[e>=0]
+        mse1 = np.sum(0.91*0.91* np.power(e1,0.66666))
+        e2 = e[e<0]
+        mse2 = np.sum(np.power(-0.7+0.57*np.power(-e2,0.3333),2))
+        mse = mse1 + mse2
+        result_list.append(mse/nsta)
+    #result_mid = np.array(list1)
+    result = np.array(result_list)
+
+    return result
+
+
+
+def pas(ob0,fo,grade_list = [0.1]):
+    '''
+
+    :param ob: 一维numpy数组
+    :param fo:  一维或者两维numpy数组，当fo是两维时，表示有多种预报进行对比
+    :return:  实数,pas评分
+    '''
+    #如果fo只包含一个预报成员
+    if len(ob0.shape) == len(fo.shape):
+        fo = fo[np.newaxis,:]
+    nfo = fo.shape[0]
+    list1 = []
+    result_list = []
+    ob_0 = ob0.flatten()
+    #对多个预报进行循环
+    for i in range(nfo):
+        nsample = 0
+        score = 0
+        foi = fo[i,:].flatten()
+        # 情况1， 实况降水u=0mm，预报降水x>0mm
+        index0 = np.where((ob_0>=grade_list[0])|(foi>=grade_list[0]))
+        #print(index0[0].size)
+
+        ob  = ob_0[index0]
+        foi = foi[index0]
+        index1 = np.where((ob == 0)&(foi>0))
+        score1 = 0
+        if index1[0].size>0:
+            fo1 = foi[index1]
+            nsample += index1[0].size
+            fo11 = fo1[fo1<5]         # 预报 0<x<5mm的情况
+            score1  += 0.5* fo11.size
+            fo12 = fo1[fo1>=5]        #预报 x>=5mm的情况
+            if fo12.size>0:
+                score1 += 0.5 * np.sum(np.exp(-(fo12 - 5) ** 2 / 25))
+            # print("情况1：")
+            # print(index1[0].size)
+            #print(score1)
+
+        # 情况2， 实况降水u>0mm，预报降水x=0mm
+        index2 = np.where((ob > 0)&(ob<10)&(foi == 0))
+        score2 = 0
+        if index2[0].size>0:
+            ob2 = ob[index2]
+            nsample += index2[0].size
+            score2 += 0.5 * np.sum(np.sin(0.05 * np.pi * (10 + ob2)))
+            # print("情况2：")
+            # print(index2[0].size)
+            #print(score2)
+
+        # 情况3， 实况降水0<u<5mm，预报降水x>0mm
+        index3 = np.where((ob>0)&(ob<5)&(foi>0))
+        score3 =0
+        if index3[0].size >0:
+            nsample += index3[0].size
+            ob3 = ob[index3]
+            fo3 = foi[index3]
+            ob31 = ob3[fo3<ob3]   #情况3.1  0<x<u,0<u<5
+            if ob31.size>0:
+                fo31 = fo3[fo3 < ob3]
+                score3 += np.sum(np.sin(0.05*np.pi*(10+ob31-fo31)))
+            ob32 = ob3[np.where((fo3<5)&(ob3<=fo3))]   #情况3.2  0<x<5,0<u<x
+            score3 += ob32.size
+            ob33 = ob3[fo3>=5]  #情况3.3  x>=5,0<u<5
+            if ob33.size>0:
+                fo33 = fo3[fo3>=5]
+                score3 += np.sum(np.exp(-(fo33-5)**2/25))
+            # print("情况3：")
+            # print(index3[0].size)
+            #print(score3)
+
+        # 情况4， 实况降水5<u<10mm，预报降水x>0mm
+        index4 = np.where((ob>=5)&(ob<10)&(foi>0))
+        score4 = 0
+        if index4[0].size>0:
+            nsample+= index4[0].size
+            ob4 = ob[index4]
+            fo4 = foi[index4]
+            ob41 = ob4[fo4<ob4]  #情况4.1 0<x<u,5<u<10
+            if ob41.size>0:
+                fo41 = fo4[fo4<ob4]
+                score4 += np.sum(np.sin(0.05*np.pi*(10+ob41-fo41)))
+            ob42 = ob4[fo4>=ob4]  #情况4.2 u<x,5<u<10
+            if ob42.size>0:
+                fo42 = fo4[fo4>=ob4]
+                score4 += np.sum(np.exp(-(fo42-ob42)**2/ob42**2))
+            # print("情况4：")
+            # print(index4[0].size)
+            #print(score4)
+
+
+        # 情况5， 实况降水u>=10mm，预报降水x>0mm
+        index5 = np.where((ob >=10)  & (foi >= 0))
+        score5 = 0
+        if index5[0].size >0:
+            nsample += index5[0].size
+            ob5 = ob[index5]
+            fo5 = foi[index5]
+            ob51 = ob5[fo5<ob5]  #情况5.1, 0<=x<u,u>=10
+            if ob51.size>0:
+                fo51 = fo5[fo5<ob5]
+                score5 += np.sum(np.sin(0.5*np.pi*fo51/ob51))
+            ob52 = ob5[fo5>=ob5]  #情况5.2, u<=x,u>=10
+            if ob52.size>0:
+                fo52 = fo5[fo5>=ob5]
+                score5 += np.sum(np.exp(-(fo52-ob52)**2/ob52**2))
+            # print("情况5：")
+            # print(index5[0].size)
+            #print(score5)
+        score = score1 + score2+score3+score4+score5
+        #print(score)
+        #计算
+        if nsample==0:
+            result1 = IV
+        else:
+            result1 = score/nsample
+        list1.append([nsample,score])
+        result_list.append(result1)
+
+    #result_mid = np.array(list1)
+    result = np.array(result_list)
+
+
+    return result
+
+
+if __name__=="__main__":
+    import pandas as pd
+    import datetime
+    import meteva
+    import xarray as xr
+
+    rain01_ob = np.random.randn(1000)
+    rain01_ob[rain01_ob < 0] = 0
+    rain01_fo = np.random.randn(2, 1000)
+    rain01_fo[rain01_fo < 0] = 0
+    print("观测和预报的平均降水强度分别是：")
+    result = ob_fo_precipitation_strength(rain01_ob, rain01_fo)  # 多个预报时 返回数组的size =  1+预报成员
+    print(result)
