@@ -1,9 +1,7 @@
 import numpy as np
 import matplotlib.pyplot as plt
-plt.rcParams['font.sans-serif']=['SimHei'] #用来正常显示中文标签\
-plt.rcParams['axes.unicode_minus']=False #用来正常显示负号
-import math
 import meteva
+import math
 import json
 
 def error_boxplot(sta_ob_and_fos0,s = None, g = None, gll=None,
