@@ -5,10 +5,9 @@ import pandas as pd
 import numpy as np
 
 
-
-
 def score(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list = None,plot = None,
-          vmax = None,vmin = None,bar_width = None,save_path = None,show = False,dpi = 300,title = "",excel_path = None,drop_g_column = False,**kwargs):
+          vmax = None,vmin = None,bar_width = None,save_path = None,show = False,dpi = 300,title = "",
+          excel_path = None,drop_g_column = False,spasify_xticks = None,keep_ob_IV = False,**kwargs):
 
 
     if s is not None:
@@ -18,8 +17,9 @@ def score(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list =
             else:
                 s["drop_last"] = True
     sta_ob_and_fos = sele_by_dict(sta_ob_and_fos0, s)
-    index = np.where(sta_ob_and_fos.iloc[:,6].values != meteva.base.IV)
-    sta_ob_and_fos = sta_ob_and_fos.iloc[index[0],:]
+    if not keep_ob_IV:
+        index = np.where(sta_ob_and_fos.iloc[:,6].values != meteva.base.IV)
+        sta_ob_and_fos = sta_ob_and_fos.iloc[index[0],:]
     if(len(sta_ob_and_fos.index) == 0):
         msg = "there is no data to verify"
         print(msg)
@@ -93,7 +93,7 @@ def score(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list =
             strs = strs.replace("lon_","")
             fo_name.append(strs)
     else:
-        ensemble_score_method = [meteva.method.cr,variance_divide_by_mse]
+        ensemble_score_method = [meteva.method.cr,meteva.method.crps,variance_divide_by_mse]
         if method in ensemble_score_method:
             fo_name = [""]
         elif method ==meteva.method.variance_mse:
@@ -112,7 +112,7 @@ def score(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list =
                       meteva.method.mr_multi, meteva.method.far_multi]
         if method in mutil_list:
         # 如果是多分类检验，但又没有设置分级方法，就需要从数据中获得全局的种类
-            values = sta_ob_and_fos.iloc[:,6:].flatten()
+            values = sta_ob_and_fos.iloc[:,6:].values.flatten()
             index_list = list(set(values))
             if len(index_list) > 30:
                 msg = "自动识别的样本类别超过30种，判断样本为连续型变量，grade_list不能缺省"
@@ -306,7 +306,7 @@ def score(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list =
                 legend = keys[1]
                 axis = keys[0]
         if "ylabel" not in plot_args.keys():
-            plot_args["ylabel"] = method.__name__.upper()
+            plot_args["ylabel"] = method.__name__
 
 
 
@@ -327,9 +327,9 @@ def score(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list =
 
         if plot is not None:
             if plot =="bar":
-                meteva.base.plot_tools.bar(result_plot,name_list_dict,legend=legend,axis = axis,vmin =vmin,vmax = vmax,bar_width=bar_width,save_path=save_path,show=show,dpi =dpi,title = title,**plot_args)
+                meteva.base.plot_tools.bar(result_plot,name_list_dict,legend=legend,axis = axis,vmin =vmin,vmax = vmax,bar_width=bar_width,save_path=save_path,show=show,dpi =dpi,title = title,spasify_xticks=spasify_xticks,**plot_args)
             else:
-                meteva.base.plot_tools.plot(result_plot,name_list_dict,legend=legend,axis = axis,vmin =vmin,vmax = vmax,save_path=save_path,show=show,dpi = dpi,title= title,**plot_args)
+                meteva.base.plot_tools.plot(result_plot,name_list_dict,legend=legend,axis = axis,vmin =vmin,vmax = vmax,save_path=save_path,show=show,dpi = dpi,title= title,spasify_xticks=spasify_xticks,**plot_args)
         if excel_path is not None:
             meteva.base.write_array_to_excel(result_plot,excel_path,name_list_dict,index= axis,columns=legend)
 
@@ -386,7 +386,7 @@ def score_id(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list
 
 
     data_names = meteva.base.get_stadata_names(sta_ob_and_fos_list[0])
-    ensemble_score_method = [meteva.method.cr, variance_divide_by_mse]
+    ensemble_score_method = [meteva.method.cr,meteva.method.crps, variance_divide_by_mse]
     if method.__name__.find("ob_fo")>=0:
         fo_name = data_names
 
@@ -525,6 +525,9 @@ def score_id(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list
                 sta_result1 = sta_merge.loc[:, coord_names]
                 sta_result1.iloc[:, -member_num:] = result[:,:,i]
                 sta_result.append(sta_result1)
+        sup_title = None
+        if "sup_title" in plot_args.keys():
+            sup_title = copy.deepcopy(plot_args["sup_title"])
 
         for i in range(len(sta_result)):
             sta_result1 = sta_result[i]
@@ -558,6 +561,8 @@ def score_id(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list
                 if plot == "scatter" or plot =="contourf":
                     if "sup_title" not in plot_args.keys():
                         plot_args["sup_title"] = ""
+                    elif isinstance(sup_title,list):
+                        plot_args["sup_title"] = sup_title[i]
                     save_path1 = None
                     if save_path is None:
                         if save_dir is None:
@@ -567,8 +572,11 @@ def score_id(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list
                             for i in range(len(title1_list)):
                                 fileName = title1_list[i].replace("\n", "").replace(":", "")
                                 save_path1.append(save_dir + "/" + fileName + ".png")
+                    elif subplot=="member":
+                        save_path1 = save_path[i]
                     else:
                         save_path1 = save_path[k * fo_num: (k + 1) * fo_num]
+
                     if plot=="scatter":
                         if "fix_size" not in plot_args.keys():
                             plot_args["fix_size"] = False
@@ -600,7 +608,7 @@ def score_id(sta_ob_and_fos0,method,s = None,g = None,gll = None,group_name_list
                         elat = elat0 + dlat0
                         grid1 = meteva.base.grid([slon,elon,dlon0],[slat,elat,dlat0])
 
-                        grd = meteva.base.interp_sg_idw_delta(sta_result1,grid1,halfR=100,nearNum=4)
+                        grd = meteva.base.interp_sg_idw_delta(sta_result1,grid1,halfR=300,nearNum=8)
                         meteva.base.tool.plot_tools.contourf_2d_grid(grd, save_path=save_path1, show=show,
                                                             title=title1_list
                                                             , add_county_line=add_county_line,
